<?php
/**
 * プラグイン作りでどのクラスからも参照する汎用クラス
 *
 * @package qa_heatmap
 */

class QAHM_Base {
	/**
	 * wordpressのget_option関数をqahm用に使いやすくした関数
	 */
	public function wrap_get_option( $option, $default = false ) {
		if ( $default === false ) {
			foreach ( QAHM_OPTIONS as $key => $value ) {
				if ( $option === $key ) {
					$default = $value;
					break;
				}
			}
			foreach ( QAHM_DB_OPTIONS as $key => $value ) {
				if ( $option === $key ) {
					$default = $value;
					break;
				}
			}
		}
		return get_option( QAHM_OPTION_PREFIX . $option, $default );
	}

	/**
	 * wordpressのupdate_option関数をqahm用に使いやすくした関数
	 */
	public function wrap_update_option( $option, $value ) {
		return update_option( QAHM_OPTION_PREFIX . $option, $value );
	}

	/**
	 * wordpressのget_user_meta関数をqahm用に使いやすくした関数
	 */
	public function wrap_get_user_meta( $user_id, $meta_key = '', $single = false ) {
		// 空文字列の場合、指定されたユーザーのすべてのメタデータを返される
		if ( $meta_key === '' ) {
			return get_user_meta( $user_id, $meta_key, $single );
		} else {
			return get_user_meta( $user_id, QAHM_OPTION_PREFIX . $meta_key, $single );
		}
	}

	/**
	 * wordpressのupdate_user_meta関数をqahm用に使いやすくした関数
	 */
	public function wrap_update_user_meta( $user_id, $meta_key, $meta_value, $prev_value = '' ) {
		return update_user_meta( $user_id, QAHM_OPTION_PREFIX . $meta_key, $meta_value, $prev_value );
	}
	
	/**
	 * phpの$this->filter_inputのラップ関数
	 */
	public function wrap_filter_input( $type, $variable_name, $filter = FILTER_DEFAULT, $options = 0 )
	{
		$checkTypes =[
			INPUT_GET,
			INPUT_POST,
			INPUT_COOKIE
		];
	
		if (in_array($type, $checkTypes) || filter_has_var($type, $variable_name)) {
			return filter_input($type, $variable_name, $filter, $options);
		} else if ($type == INPUT_SERVER && isset($_SERVER[$variable_name])) {
			return filter_var($_SERVER[$variable_name], $filter, $options);
		} else if ($type == INPUT_ENV && isset($_ENV[$variable_name])) {
			return filter_var($_ENV[$variable_name], $filter, $options);
		} else {
			return null;
		}
	}

	/**
	 * wordpressのwp_mailをベースにQAから送れるようにしたもの
	 */
	public function qa_mail( $subject, $message ) {
		global $qahm_data_api;
		$homeurl = get_home_url();
		$domain  = parse_url( $homeurl, PHP_URL_HOST );
		$from    = 'wordpress@' . $domain;
		$return  = false;

		$headers = array("From: QA <{$from}>", "Content-Type: text/plain; charset=UTF-8");
		$eary = $qahm_data_api->get_email_notice_ary();
		if ( array_key_exists('regist', $eary ) ) {
			if ( $eary['regist'] ) {
				$to = $eary['email'];
				$return = wp_mail( $to, $subject, $message, $headers );
			}
		}
		return $return;
	}

	/**
	 * アクセス権限判定
	 */
	public function check_access_role() {
		if ( ! is_user_logged_in() ) {
			return false;
		}
		$user   = wp_get_current_user();
		$roles  = (array) $user->roles;
		$role   = $roles[0];
		$access = $this->wrap_get_option( 'access_role', 'administrator' );

		switch ( $role ) {
			case 'administrator':
				return ( 'administrator' === $access || 'editor' === $access ) ? true : false;
			case 'editor':
				return ( 'editor' === $access ) ? true : false;
			default:
				return false;
		}
	}

	/**
	 * ライセンスプラン配列を取得
	 * プラグインver3.0以降の新プランに対応
	 */
	public function get_plan() {
		$plans = $this->wrap_get_option( 'license_plans' );
		if ( ! $plans ) {
			return null;
		}
		return json_decode( $plans, true );
	}

	/**
	 * 有料メンバーかどうかの判別（画面表示用）//yuko仮
	 */
	public function is_subscribed() {
		$plans = $this->wrap_get_option( 'license_plans' );
		if ( is_null($plans) ) {
			$plan = $this->wrap_get_option( 'license_plan' );
			if ( ! $plan ) {
				return false;
			} elseif ( $plan == 'friend' ) {
				return false;
			}
		}
		return true;
	}	

	
	/**
	* 該当のプランが組み込まれているかチェック
	* 組み込まれていればその値を返す
	*/
   public function check_plan( $plan_name ) {
	   $plans = $this->get_plan();

	   if ( $plans && array_key_exists( $plan_name, $plans ) ) {
		   return $plans[$plan_name];
	   } else {
		   return false;
	   }
   }

	/**
	 * ユーザーが課金しているか判定
	 */
/*	public function check_paid( $plan_name ) {
		$plans = $this->wrap_get_option( 'license_plans' );
		if ( ! $plans ) {
			return false;
		}

		if ( array_key_exists( $plan_name, $plans ) ) {
			return $plans[$plan_name];
		}
	}
*/
	/**
	 * プラグインのメインファイルパスを取得
	 */
	public function get_plugin_main_file_path() {
		return dirname( __FILE__ ) . '/' . QAHM_NAME . '.php';
	}

	/**
	 * jsディレクトリのパスを取得
	 */
	public function get_js_dir_path() {
		return plugin_dir_path( __FILE__ ) . 'js/';
	}

	/**
	 * jsディレクトリのURLを取得
	 */
	public function get_js_dir_url() {
		return plugin_dir_url( __FILE__ ) . 'js/';
	}

	/**
	 * cssディレクトリのパスを取得
	 */
	public function get_css_dir_path() {
		return plugin_dir_path( __FILE__ ) . 'css/';
	}

	/**
	 * cssディレクトリのURLを取得
	 */
	public function get_css_dir_url() {
		return plugin_dir_url( __FILE__ ) . 'css/';
	}

	/**
	 * imgディレクトリのパスを取得
	 */
	public function get_img_dir_path() {
		return plugin_dir_path( __FILE__ ) . 'img/';
	}

	/**
	 * imgディレクトリのURLを取得
	 */
	public function get_img_dir_url() {
		return plugin_dir_url(__FILE__) . 'img/';
	}

	/**
	 * dataディレクトリのパスを取得
	 * 引数にdataディレクトリからのパスを入力することにより、
	 * dataディレクトリからの相対パスを取得することができる。
	 * 
	 * なおこの関数では念のためディレクトリの存在チェック＆mkdirも行うが、
	 * 相対パスに深い階層を指定してもmkdirされるのは最後の階層のみであり
	 * 道中の階層にはmkdirされない。その点には注意
	 */
	public function get_data_dir_path( $data_rel_path = '' ) {
		global $wp_filesystem;
		$path = $wp_filesystem->wp_content_dir() . QAHM_TEXT_DOMAIN . '-data/';
		if ( ! $wp_filesystem->exists( $path ) ) {
			$wp_filesystem->mkdir( $path );
		}

		if ( $data_rel_path ) {
			$path .= $data_rel_path;
			if ( substr( $path, -1 ) !== '/' ) {
				$path .= '/';
			}

			if ( ! $wp_filesystem->exists( $path ) ) {
				$wp_filesystem->mkdir( $path );
			}
		}

		return $path;
	}

	/**
	 * dataディレクトリのURLを取得
	 */
	public function get_data_dir_url( $data_rel_path = '' ) {
		$path = content_url() . '/' . QAHM_TEXT_DOMAIN . '-data/';

		if ( $data_rel_path ) {
			$path .= $data_rel_path;
			if ( substr( $path, -1 ) !== '/' ) {
				$path .= '/';
			}
		}

		return $path;
	}

	/**
	 * tempディレクトリのパスを取得
	 */
	public function get_temp_dir_path() {
		return plugin_dir_path( __FILE__ ) . 'temp/';
	}

	/**
	 * cronのロックファイルのパスを取得
	 */
	public function get_cron_lock_path() {
		return $this->get_data_dir_path() . 'cron_lock';
	}

	/**
	 * cronのステータスファイルのパスを取得 -- maruyama
	 */
	public function get_cron_status_path() {
		return $this->get_data_dir_path() . 'cron_status';
	}

	/**
	 * cronのバックアップファイルのパスを取得 -- maruyama
	 */
	public function get_cron_backup_path() {
		return $this->get_data_dir_path() . 'cron_backup';
	}

	/**
	 * プラグインのメンテナンスモードか判定
	 */
	public function is_maintenance() {
		global $wp_filesystem;
		$maintenance_path = $this->get_temp_dir_path() . 'maintenance.php';
		if ( $wp_filesystem->exists( $maintenance_path ) ) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * プラグインのemail noticeの登録情報を判定
	 */
	public function is_email_regist() {
		$json = $this->wrap_get_option('email_notice');
		if ( $json !== '' && $json !== false ) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * トラッキングIDを取得
	 */
	public function get_tracking_id( $url = null ) {
		if ( $this->is_wordpress() ) {
			// auto
			$parse_url = parse_url( get_home_url() );
			$id = 'a&' . $parse_url['host'];
		} else {
			// manual
			$parse_url = parse_url( $url );
			$id = 'm&' . $parse_url['host'];
		}
		return hash( 'fnv164', $id );
	}

	/**
	 * WPサイトか判定
	 * この判定方法で良いのかは要検証。今後変わる可能性あり
	 */
	public function is_wordpress() {
		if ( function_exists( 'wp_nonce_field' ) ) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * qahm対象ページか判定
	 * ※プラグイン読み込み直後のコンストラクタやwp_ajax内では$type引数指定無しの形は使えないので注意
	 */
	public function is_qahm_page( $type = null ) {
		if ( $type ){
			if(
				$type === 'home' ||
				$type === 'page_id' ||
				$type === 'p' ||
				$type === 'cat' ||
				$type === 'tag' ||
				$type === 'tax'
			) {
				return true;
			} else {
				return false;
			}
		} else {
			if ( is_home() || is_front_page() || is_page() || is_single() || is_category() || is_tag() || is_tax() ) {
				return true;
			} else {
				return false;
			}
		}
	}

	/**
	 * ajaxに関数を登録
	 */
	public function regist_ajax_func( $func ) {
		add_action( 'wp_ajax_' . QAHM_NAME . '_' . $func, array( $this, $func ) );
		add_action( 'wp_ajax_nopriv_' . QAHM_NAME . '_' . $func, array( $this, $func ) );
	}
	
	/**
	 * wp_filesystem 初期化
	 */
	public function init_wp_filesystem() {
		require_once ABSPATH . 'wp-admin/includes/file.php';
		global $wp_filesystem;

		if ( empty( $wp_filesystem ) ) {
			$creds = false;

			$access_type = get_filesystem_method();
			if ( $access_type === 'ftpext' ) {
				$creds = request_filesystem_credentials( '', '', false, false, null );
			}
			if ( ! WP_Filesystem( $creds ) ) {
				http_response_code(503);
				throw new Exception( 'Service Temporarily Unavailable. WP_Filesystem Invalid Credentials' );
			}
		}
	}

	/**
	 * uriエンコードし小文字にして返す
	 */
	public function encode_uri( $uri ) {
		$uri = preg_replace_callback( "{[^0-9a-z_.!~*'();,/?:@&=+$#-]}i", function ($m) {
			return sprintf('%%%02X', ord($m[0]));
		}, $uri );
		return mb_strtolower( $uri );
	}
	
	/**
	 * bot判定
	 */
	public function is_bot() {
		$bot = array(
			'Googlebot',
			'msnbot',
			'bingbot',
			'Yahoo! Slurp',
			'Y!J',
			'facebookexternalhit',
			'Twitterbot',
			'Applebot',
			'Linespider',
			'Baidu',
			'YandexBot',
			'Yeti',
			'dotbot',
			'rogerbot',
			'AhrefsBot',
			'MJ12bot',
			'SMTBot',
			'BLEXBot',
			'linkdexbot',
			'SemrushBot',
			'360Spider',
			'spider',
			'YoudaoBot',
			'DuckDuckGo',
			'Daum',
			'Exabot',
			'SeznamBot',
			'Steeler',
			'Sonic',
			'BUbiNG',
			'Barkrowler',
			'GrapeshotCrawler',
			'MegaIndex.ru',
			'archive.org_bot',
			'TweetmemeBot',
			'PaperLiBot',
			'admantx-apacas',
			'SafeDNSBot',
			'TurnitinBot',
			'proximic',
			'ICC-Crawler',
			'Mappy',
			'YaK',
			'CCBot',
			'Pockey',
			'psbot',
			'Feedly',
			'Superfeedr bot',
			'ltx71',
			'Mail.RU_Bot',
			'Linguee Bot',
			'DuckDuckBot',
			'bidswitchbot',
			'applebot',
			'istellabot',
			'integralads',
			'jet-bot',
			'trendictionbot',
			'blogmuraBot',
			'NetSeer crawler',
			QAHM_NAME . 'bot',
		);

		// 正規表現用に配列を置換
		for ( $i = 0, $bot_len = count( $bot ); $i < $bot_len; $i++ ) {
			$bot[ $i ] = str_replace( '.', '\.', $bot[ $i ] );
			$bot[ $i ] = str_replace( '-', '\-', $bot[ $i ] );
		}

		$ua = $this->wrap_filter_input( INPUT_SERVER, 'HTTP_USER_AGENT' );
		if( $ua ) {
			if ( preg_match( '/' . implode( '|', $bot ) . '/', $ua ) ) {
				return true;
			}
		}
		return false;
	}

	public function os_from_ua( $ua ) {
		$device_os = '';
		if ( preg_match( '/(iPhone|iPod|iPad|Windows Phone|Opera Mobi|Fennec|Android)/', $ua, $match ) ) {
			$device_os = $match[1];
		} else {
			if ( preg_match( '/(Mac OS X [0-9]*.[0-9]*|Windows NT [0-9]*.[0-9]*)/', $ua, $match ) ) {
				$device_os = $match[1];
			}
		}
		$device_os = str_replace( '_', '.', $device_os );
		return $device_os;
	}

	public function browser_from_ua( $ua ) {
		$browser = '';
		$version = '';
		// ブラウザの判別。
		if ( preg_match( '/(MSIE|Chrome|Firefox|Android|Safari|Opera|jp.co.yahoo.ipn.appli)[\/ ]([0-9.]*)/', $ua, $match ) ) {
			$browser = $match[1];
			$version = $match[2];
		}
		return $browser . '/' . $version;
	}

	public function is_zip ($string) {
		$is_zip = false;
		if ( 3 < strlen( $string ) ) {
			$byte1 = strtoupper( bin2hex( substr( $string,0,1 ) ) );
			$byte2 = strtoupper( bin2hex( substr( $string,1,1 ) ) );
			$byte3 = strtoupper( bin2hex( substr( $string,2,1 ) ) );

			if( $byte1=="1F" && $byte2=="8B" && $byte3=="08" ){
				$is_zip = true;
			}
		}
		return $is_zip;
	}

	/**
	 * qa 翻訳関数
	 */
	public function japan( $text, $domain = '' ) {
		return $text;
	}
}
